import { AsyncLocalStorage } from "async_hooks";
import { DOMContainer } from "@microsoft/fast-element/di.js";
declare function getItem<T = any>(key: any, fallback: () => T): T;
declare function getItem<T = any>(key: any): T | undefined;
/**
 * Get and set values that are local to the current
 * HTTP request.
 * @remarks
 * Also, provides access to a request-scoped DI container.
 * @beta
 */
export declare const RequestStorage: Readonly<{
    /**
     * The request-scoped DI container.
     */
    readonly container: DOMContainer;
    /**
     * Stores a value in request-scoped storage.
     * @param key - The key to store the value under.
     * @param value - The value to store.
     * @returns This RequestStorage instance.
     */
    set(key: any, value: any): any;
    /**
     * Gets a value from request-scoped storage.
     * @param key - The key to get the value for.
     * @param fallback - A function that provides a value if there is nothing stored under the key.
     * @returns The value stored under the key.
     * @remarks
     * The return result of the fallback function will be stored under the key for later access.
     */
    get: typeof getItem;
    /**
     * Clears all request-scoped values.
     */
    clear(): void;
    /**
     *
     * @param key - The request-scoped key to delete.
     * @returns true if the value was deleted; false otherwise.
     */
    delete(key: any): boolean;
    /**
     * Determines whether there's a request-scoped value for the given key.
     * @param key - The key to check.
     * @returns true if the key exists; false otherwise
     */
    has(key: any): boolean;
}>;
/**
 * Options used in creating the backing storage for RequestStorage.
 * @beta
 */
export declare type StorageOptions = {
    /**
     * A custom window creation function.
     */
    createWindow?: () => {
        [key: string]: unknown;
    };
    /**
     * Initial values to setup in the backing store.
     */
    storage?: Map<any, any>;
};
/**
 * An Express-compatible middleware function.
 * @beta
 */
export declare type Middleware = (req: any, res: any, next: () => any) => void;
/**
 * APIs used in configuring and managing RequestStorage.
 * @beta
 */
export declare const RequestStorageManager: Readonly<{
    /**
     * Gets the current AsyncLocalStorage instance that provides
     * the backend for the RequestStorageManager.
     */
    backend: AsyncLocalStorage<unknown>;
    /**
     * Installs a DOM shim that ensures that window, document,
     * and other globals are scoped per-request. Calling this function
     * will have no effect if the shim has already been installed.
     *
     * @throws TypeError when properties cannot be defined on the globalThis.
     */
    installDOMShim(): void;
    /**
     * Uninstalls the DOM shim installed by {@link RequestStorageManager.installDOMShim}.
     * Calling this function will have no effect if there is no shim installed.
     */
    uninstallDOMShim(): void;
    /**
     * Installs the dependency injection system as the strategy for
     * handling Context requests. The installed behavior will use
     * RequestStorage.container as the root DI container.
     */
    installDIContextRequestStrategy(): void;
    /**
     * Creates a backing store for RequestStorage.
     * @param options - The options used when creating the backing store for RequestStorage.
     * @returns A Map suitable as a backing store for RequestStorage.
     */
    createStorage(options?: StorageOptions): Map<any, any>;
    /**
     * Runs a function with the provided storage available through RequestStorage.
     * @param storage - The storage to scope to the callback.
     * @param callback - The function to execute with the storage context.
     * @returns The result of the invoked function.
     */
    run<T = unknown>(storage: Map<any, any>, callback: () => T): T;
    /**
     * Creates an Express-compatible middleware function.
     * @param options - The options used when creating the backing store for RequestStorage.
     * @returns A middleware function.
     * @remarks
     * Invoking this function installs the RequestStorageManager DOM shim and then returns
     * an Express-compatible middleware function. The middleware ensures that a unique
     * storage instance is created for every request and that the following middleware
     * functions in the pipeline are processed in the context of this storage.
     */
    middleware(options?: StorageOptions): Middleware;
}>;
export {};
