import { ExecutionContext, ViewTemplate } from "@microsoft/fast-element";
import { RenderInfo } from "../render-info.js";
import { AsyncElementRenderer, ConstructableElementRenderer } from "../element-renderer/interfaces.js";
import { Op } from "../template-parser/op-codes.js";
import { ViewBehaviorFactoryRenderer } from "./directives.js";
/** @beta */
export interface TemplateRenderer {
    render(template: ViewTemplate | string, renderInfo?: RenderInfo, source?: unknown, context?: ExecutionContext): IterableIterator<string>;
    createRenderInfo(): RenderInfo;
    withDefaultElementRenderers(...renderers: ConstructableElementRenderer[]): void;
}
/** @beta */
export interface AsyncTemplateRenderer {
    render(template: ViewTemplate | string, renderInfo?: RenderInfo, source?: unknown, context?: ExecutionContext): IterableIterator<string | Promise<string>>;
    createRenderInfo(): RenderInfo;
    withDefaultElementRenderers(...renderers: ConstructableElementRenderer<AsyncElementRenderer>[]): void;
}
/**
 * A class designed to render HTML templates. The renderer supports
 * rendering {@link @microsoft/fast-element#ViewTemplate} instances as well
 * as arbitrary HTML strings.
 *
 * @internal
 */
export declare class DefaultTemplateRenderer implements TemplateRenderer {
    private viewBehaviorFactoryRenderers;
    private defaultElementRenderers;
    /**
     * Renders a {@link @microsoft/fast-element#ViewTemplate} or HTML string.
     * @param template - The template to render.
     * @param renderInfo - Information about the rendering context.
     * @param source - Any source data to render the template and evaluate bindings with.
     * @param context - The {@link @microsoft/fast-element#ExecutionContext} to render with.
     */
    render(template: ViewTemplate | string, renderInfo?: RenderInfo, source?: unknown, context?: ExecutionContext): IterableIterator<string>;
    /**
     * Render a set of op codes.
     * @param codes - the op codes to render.
     * @param renderInfo - renderInfo context.
     * @param source - source data.
     *
     * @internal
     */
    renderOpCodes(codes: Op[], renderInfo: RenderInfo, source: unknown, context: ExecutionContext): IterableIterator<string>;
    /**
     * Constructs a new {@link RenderInfo } object.
     * @param renderers - the ElementRenderer constructors the RenderInfo should contain
     * @returns
     */
    createRenderInfo(renderers?: ConstructableElementRenderer[]): RenderInfo;
    /**
     * Configures the ElementRenderers used during RenderInfo construction by {@link DefaultTemplateRenderer.createRenderInfo}
     * and the default RenderInfo argument used by {@link DefaultTemplateRenderer.render}.
     * @param renderers - The ElementRenderers to use by default.
     */
    withDefaultElementRenderers(...renderers: ConstructableElementRenderer[]): void;
    /**
     * Registers DirectiveRenderers to use when rendering templates.
     * @param renderers - The directive renderers to register
     *
     * @internal
     */
    withViewBehaviorFactoryRenderers(...renderers: ViewBehaviorFactoryRenderer<any>[]): void;
    private getAttributeBindingRenderer;
    /**
     * Format attribute key/value pair into a HTML attribute string.
     * @param name - the attribute name.
     * @param value - the attribute value.
     */
    private static formatAttribute;
    /**
     * Renders an attribute binding
     */
    private static renderAttribute;
    /**
     * Renders a property or tokenList binding
     */
    private static renderProperty;
    /**
     * Renders a boolean attribute binding
     */
    private static renderBooleanAttribute;
}
