/**
 * This file was heavily inspired by {@link https://github.com/lit/lit/tree/main/packages/labs/ssr} with some adjustments made.
 * Please see {@link ../ACKNOWLEDGEMENTS.md}
 */
import { shouldBubble } from "./event-utilities.js";
/**
 * @beta
 */
export class Node extends EventTarget {
}
/**
 * @beta
 */
export class Element extends Node {
}
function checkTokenAndGetIndex(classList, token) {
    if (token === "") {
        throw new Error("The token must not be empty.");
    }
    if (/\s/.test(token)) {
        throw new Error("The token must not contain space characters.");
    }
    return classList.indexOf(token);
}
/*
 * Based on classList.js: Cross-browser full element.classList implementation.
 * 1.2.20171210
 *
 * Credit to Eli Grey, http://eligrey.com
 * License: Dedicated to the public domain.
 *   See https://github.com/eligrey/classList.js/blob/master/LICENSE.md
 */
/**
 * @beta
 */
export class DOMTokenList {
    constructor(element, attributeName) {
        this.element = element;
        this.attributeName = attributeName;
        this.tokens = [];
        const trimmedClasses = (element.getAttribute(attributeName) || "").trim();
        const classes = trimmedClasses ? trimmedClasses.split(/\s+/) : [];
        for (let i = 0, len = classes.length; i < len; i++) {
            this.tokens.push(classes[i]);
        }
    }
    item(i) {
        return this.tokens[i] || null;
    }
    contains(token) {
        return ~checkTokenAndGetIndex(this, token + "");
    }
    indexOf(token) {
        return this.tokens.indexOf(token);
    }
    add(...tokens) {
        const length = tokens.length;
        let index = 0;
        let updated = false;
        do {
            const token = tokens[index] + "";
            if (!~checkTokenAndGetIndex(this, token)) {
                this.tokens.push(token);
                updated = true;
            }
        } while (++index < length);
        if (updated) {
            this.updateClassName();
        }
    }
    remove(...tokens) {
        const length = tokens.length;
        let index = 0;
        let updated = false;
        do {
            const token = tokens[index] + "";
            let result = checkTokenAndGetIndex(this, token);
            while (~result) {
                this.tokens.splice(result, 1);
                updated = true;
                result = checkTokenAndGetIndex(this, token);
            }
        } while (++index < length);
        if (updated) {
            this.updateClassName();
        }
    }
    toggle(token, force) {
        const result = this.contains(token);
        const method = result ? force !== true && "remove" : force !== false && "add";
        if (method) {
            this[method](token);
        }
        if (force === true || force === false) {
            return force;
        }
        else {
            return !result;
        }
    }
    replace(token, replacement) {
        const index = checkTokenAndGetIndex(this, token + "");
        if (~index) {
            this.tokens.splice(index, 1, replacement);
            this.updateClassName();
        }
    }
    toString() {
        return this.tokens.join(" ");
    }
    updateClassName() {
        this.element.setAttribute(this.attributeName, this.toString());
    }
}
/**
 * @beta
 */
export class HTMLElement extends Element {
    constructor() {
        super(...arguments);
        this._shadowRoot = null;
    }
    static getOrCreateAttributesForElement(element) {
        let attrs = HTMLElement.elementAttributes.get(element);
        if (!attrs) {
            HTMLElement.elementAttributes.set(element, (attrs = new Map()));
        }
        return attrs;
    }
    get attributes() {
        return Array.from(HTMLElement.getOrCreateAttributesForElement(this)).map(([name, value]) => ({
            name,
            value,
        }));
    }
    get shadowRoot() {
        return this._shadowRoot;
    }
    get part() {
        return new DOMTokenList(this, "part");
    }
    get classList() {
        return new DOMTokenList(this, "class");
    }
    get className() {
        return this.getAttribute("class");
    }
    set className(value) {
        if (value === null) {
            this.removeAttribute("class");
        }
        else {
            this.setAttribute("class", value);
        }
    }
    setAttribute(name, value) {
        HTMLElement.getOrCreateAttributesForElement(this).set(name, value);
    }
    removeAttribute(name) {
        HTMLElement.getOrCreateAttributesForElement(this).delete(name);
    }
    hasAttribute(name) {
        return HTMLElement.getOrCreateAttributesForElement(this).has(name);
    }
    attachShadow(init) {
        const shadowRoot = { host: this };
        if (init && init.mode === "open") {
            this._shadowRoot = shadowRoot;
        }
        return shadowRoot;
    }
    getAttribute(name) {
        const value = HTMLElement.getOrCreateAttributesForElement(this).get(name);
        return value === undefined ? null : value;
    }
}
HTMLElement.elementAttributes = new WeakMap();
/**
 * @beta
 */
export class ShadowRoot {
}
/**
 * @beta
 */
export class Document extends Node {
    constructor() {
        super(...arguments);
        this.adoptedStyleSheets = [];
    }
    createTreeWalker() {
        return {};
    }
    createTextNode() {
        return {};
    }
    createElement() {
        return {};
    }
    dispatchEvent(event) {
        let canceled = super.dispatchEvent(event);
        if (shouldBubble(event)) {
            canceled = window.dispatchEvent(event);
        }
        return canceled;
    }
}
/**
 * @beta
 */
export class CustomEvent extends Event {
    constructor(type, init) {
        super(type, init);
        this.detail = null;
        if (init && "detail" in init) {
            this.detail = init.detail;
        }
    }
}
/**
 * @beta
 */
export class CSSStyleRule {
    constructor() {
        this.style = new CSSStyleDeclaration();
    }
    get cssText() {
        if (typeof this.selectorText !== "string") {
            throw new Error("Cannot format cssText for CSSStyleRule. No selectorText property is assigned.");
        }
        return `${this.selectorText} { ${this.style.cssText} }`;
    }
}
/**
 * @beta
 */
export class CSSStyleDeclaration {
    constructor() {
        this.rules = new Map();
    }
    setProperty(name, value = "") {
        this.rules.set(name, value);
    }
    removeProperty(name) {
        this.rules.delete(name);
    }
    get cssText() {
        let text = "";
        for (const [key, value] of this.rules) {
            text += `${key}: ${value};`;
        }
        return text;
    }
}
/**
 * @beta
 */
export class CSSStyleSheet {
    constructor() {
        this.cssRules = [];
    }
    replace() { }
    insertRule(rule, index = 0) {
        const selectorText = rule.split("{")[0];
        const styleRule = new window.CSSStyleRule();
        styleRule.selectorText = selectorText;
        this.cssRules.splice(index, 0, styleRule);
        return index;
    }
}
/**
 * @beta
 */
export class CustomElementRegistry {
    constructor() {
        this.__definitions = new Map();
    }
    define(name, ctor) {
        var _a;
        this.__definitions.set(name, {
            ctor,
            observedAttributes: (_a = ctor.observedAttributes) !== null && _a !== void 0 ? _a : [],
        });
    }
    get(name) {
        const definition = this.__definitions.get(name);
        return definition && definition.ctor;
    }
}
/**
 * @beta
 */
export class MutationObserver {
    observe() { }
}
/**
 * Shim of MediaQueryList.
 * {@link https://developer.mozilla.org/en-US/docs/Web/API/MediaQueryList} */
export class MediaQueryList {
    constructor() {
        /** Always false */
        this.matches = false;
    }
    /** No-op */
    addEventListener() { }
}
/**
 * @beta
 */
export class Window extends EventTarget {
    constructor(props) {
        super();
        this.Node = Node;
        this.Element = Element;
        this.HTMLElement = HTMLElement;
        this.Document = Document;
        this.CustomEvent = CustomEvent;
        this.CSSStyleSheet = CSSStyleSheet;
        this.CSSStyleDeclaration = CSSStyleDeclaration;
        this.CSSStyleRule = CSSStyleRule;
        this.ShadowRoot = ShadowRoot;
        this.CustomElementRegistry = CustomElementRegistry;
        this.MutationObserver = MutationObserver;
        this.MediaQueryList = MediaQueryList;
        this.matchMedia = () => new this.MediaQueryList();
        /**
         * Methods of EventTarget must be assigned explicitly, otherwise they get omitted
         * when the window is merged into the `globalThis` in {@link installWindowOnGlobal}.
         */
        this.dispatchEvent = EventTarget.prototype.dispatchEvent.bind(this);
        this.addEventListener = EventTarget.prototype.addEventListener.bind(this);
        this.removeEventListener = EventTarget.prototype.removeEventListener.bind(this);
        Object.assign(this, props);
        this.window = this;
        this.document = new this.Document();
        this.customElements = new this.CustomElementRegistry();
    }
}
/**
 * Creates a window object.
 * @param props - Additional properties to expose on the window.
 *
 * @beta
 */
export function createWindow(props) {
    return new Window(props);
}
/**
 * Installs a window object on the global, exposing properties directly on globalThis and globalthis.window
 * @param window - The Window object to install
 *
 * @beta
 */
export function installWindowOnGlobal(window) {
    if (globalThis.window === undefined) {
        Object.assign(globalThis, window);
        globalThis.window = globalThis;
    }
}
