import { __decorate, __metadata } from "tslib";
/**
 * This file was ported from {@link https://github.com/lit/lit/tree/main/packages/labs/ssr}.
 * Please see {@link ../ACKNOWLEDGEMENTS.md}
 */
import { observable } from "@microsoft/fast-element";
import { escapeHtml } from "../escape-html.js";
import { HTMLElement as ShimHTMLElement } from "../dom-shim.js";
import { shouldBubble } from "../event-utilities.js";
export const getElementRenderer = (renderInfo, tagName, ceClass = customElements.get(tagName), attributes = new Map()) => {
    if (ceClass === undefined) {
        console.warn(`Custom element ${tagName} was not registered.`);
    }
    else {
        for (const renderer of renderInfo.elementRenderers) {
            if (renderer.matchesClass(ceClass, tagName, attributes)) {
                return new renderer(tagName, renderInfo);
            }
        }
    }
    return new FallbackRenderer(tagName, renderInfo);
};
/**
 * @beta
 */
export class DefaultElementRenderer {
    constructor(tagName, renderInfo) {
        this.tagName = tagName;
        this.renderInfo = renderInfo;
        this.parent = null;
        this.parent = (renderInfo === null || renderInfo === void 0 ? void 0 : renderInfo.customElementInstanceStack.at(-1)) || null;
    }
    elementChanged() {
        if (this.element) {
            const dispatch = this.element.dispatchEvent;
            Reflect.defineProperty(this.element, "dispatchEvent", {
                value: (event) => {
                    let canceled = dispatch.call(this.element, event);
                    if (shouldBubble(event)) {
                        if (this.parent) {
                            canceled = this.parent.dispatchEvent(event);
                        }
                        else if (this.element instanceof ShimHTMLElement) {
                            // Only emit on document if the the element is the DOM shim's element.
                            // Otherwise, the installed DOM should implement it's own behavior for bubbling
                            // an event from an element to the document and window.
                            canceled = document.dispatchEvent(event);
                        }
                    }
                    return canceled;
                },
            });
        }
    }
    /**
     * Should return true when the renderer should handle rendering for a custom element ctor or tag name.
     * @param ctor - the custom element class
     * @param tagName - the custom element tag name
     * @param attributes - the attribute map
     * @returns
     */
    static matchesClass(ctor, tagName, attributes) {
        return false;
    }
    dispatchEvent(event) {
        var _a;
        return ((_a = this.element) === null || _a === void 0 ? void 0 : _a.dispatchEvent(event)) || false;
    }
    /**
     * Sets a property for the element if it exists.
     */
    setProperty(name, value) {
        if (this.element) {
            this.element[name] = value;
        }
    }
    /**
     * Sets an attribute for the element if it exists.
     */
    setAttribute(name, value) {
        if (this.element) {
            const prev = this.element.getAttribute(name);
            this.element.setAttribute(name, value);
            this.attributeChangedCallback(name, prev, value);
        }
    }
}
__decorate([
    observable,
    __metadata("design:type", HTMLElement)
], DefaultElementRenderer.prototype, "element", void 0);
/**
 * An ElementRenderer used as a fallback in the case where a custom element is
 * either unregistered or has no other matching renderer.
 */
class FallbackRenderer extends DefaultElementRenderer {
    constructor() {
        super(...arguments);
        this._attributes = {};
    }
    setAttribute(name, value) {
        this._attributes[name] = value;
    }
    *renderAttributes() {
        for (const [name, value] of Object.entries(this._attributes)) {
            if (value === "" || value === undefined || value === null) {
                yield ` ${name}`;
            }
            else {
                yield ` ${name}="${escapeHtml(value)}"`;
            }
        }
    }
    connectedCallback() { }
    attributeChangedCallback() { }
    /* eslint-disable-next-line */
    *renderShadow() { }
}
