import { __awaiter } from "tslib";
import { DOM, DOMAspect, ExecutionContext, FASTElement } from "@microsoft/fast-element";
import { PendingTaskEvent } from "@microsoft/fast-element/pending-task.js";
import { escapeHtml } from "../escape-html.js";
import { FASTSSRStyleStrategy } from "../styles/style-strategy.js";
import { SSRView } from "../view.js";
import { DefaultElementRenderer } from "./element-renderer.js";
/**
 * An {@link ElementRenderer} implementation designed to render components
 * built with FAST.
 *
 */
class FASTElementRenderer extends DefaultElementRenderer {
    /**
     * Constructs a new {@link FASTElementRenderer}.
     * @param tagName - the tag-name of the element to create.
     */
    constructor(tagName, renderInfo) {
        super(tagName, renderInfo);
        const ctor = customElements.get(this.tagName);
        if (ctor) {
            this.element = new ctor();
            this.element.tagName = tagName;
        }
        else {
            throw new Error(`FASTElementRenderer was unable to find a constructor for a custom element with the tag name '${tagName}'.`);
        }
    }
    static matchesClass(ctor, tagName, attributes) {
        return ctor.prototype instanceof FASTElement;
    }
    /**
     * Indicate to the {@link FASTElementRenderer} that the instance should execute DOM connection behavior.
     */
    connectedCallback() {
        this.element.connectedCallback();
        const view = this.element.$fastController.view;
        if (view && SSRView.isSSRView(view)) {
            if (view.hostStaticAttributes) {
                view.hostStaticAttributes.forEach((value, key) => {
                    this.element.setAttribute(key, value);
                });
            }
            if (view.hostDynamicAttributes) {
                for (const attr of view.hostDynamicAttributes) {
                    const result = attr.dataBinding.evaluate(this.element, ExecutionContext.default);
                    const { target } = attr;
                    switch (attr.aspect) {
                        case DOMAspect.property:
                            this.element[target] = result;
                            break;
                        case DOMAspect.attribute:
                            DOM.setAttribute(this.element, target, result);
                            break;
                        case DOMAspect.booleanAttribute:
                            DOM.setBooleanAttribute(this.element, target, result);
                            break;
                    }
                }
            }
        }
    }
    /**
     * Indicate to the {@link FASTElementRenderer} that an attribute has been changed.
     * @param name - The name of the changed attribute
     * @param old - The old attribute value
     * @param value - The new attribute value
     */
    attributeChangedCallback(name, old, value) {
        this.element.attributeChangedCallback(name, old, value);
    }
}
export class SyncFASTElementRenderer extends FASTElementRenderer {
    constructor() {
        super(...arguments);
        this.renderAttributes = renderAttributesSync;
        this.renderShadow = renderShadow;
    }
}
export class AsyncFASTElementRenderer extends FASTElementRenderer {
    constructor(tagName, renderInfo) {
        super(tagName, renderInfo);
        this.renderShadow = renderShadow;
        this.awaiting = new Set();
        this.pendingTaskHandler = (e) => {
            if (PendingTaskEvent.isPendingTask(e)) {
                this.awaiting.add(e.complete);
            }
        };
        this.element.addEventListener(PendingTaskEvent.type, this.pendingTaskHandler);
    }
    *renderAttributes() {
        if (this.element !== undefined) {
            if (this.awaiting.size) {
                yield this.pauseRendering().then(() => "");
            }
            yield* renderAttributesSync.call(this);
        }
    }
    pauseRendering() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const awaiting of this.awaiting) {
                try {
                    yield awaiting;
                }
                catch (e) {
                    // Await will throw if the Promise is rejected. In that case,
                    // SSR should just continue rendering
                }
            }
            this.awaiting.clear();
        });
    }
}
function* renderAttributesSync() {
    if (this.element !== undefined) {
        const { attributes } = this.element;
        for (let i = 0, name, value; i < attributes.length && ({ name, value } = attributes[i]); i++) {
            if (value === "" || value === undefined || value === null) {
                yield ` ${name}`;
            }
            else if (typeof value === "string") {
                yield ` ${name}="${escapeHtml(value)}"`;
            }
            else if (typeof value === "boolean") {
                yield ` ${name}="${value}"`;
            }
            else {
                throw new Error(`Cannot assign attribute '${name}' for element ${this.element.tagName}.`);
            }
        }
        if (this.deferHydration) {
            yield " defer-hydration";
        }
    }
}
function* renderShadow(renderInfo) {
    const view = this.element.$fastController.view;
    const styles = FASTSSRStyleStrategy.getStylesFor(this.element);
    if (styles) {
        for (const style of styles) {
            yield this.styleRenderer.render(style);
        }
    }
    if (view !== null) {
        yield* this.templateRenderer.renderOpCodes(view.codes, renderInfo, this.element, ExecutionContext.default);
    }
}
