/**
 * The FASTStyle web component that takes attributes:
 * - css - a string version of the CSS to be applied to the parent web component, this can be omitted if the style-id has been used in the DOM as the cached value will be fetched
 * - style-id - a dataset attribute used as an identifier for the CSS attr string
 */
export default class FASTStyle extends HTMLElement {
    constructor() {
        super(...arguments);
        /**
         * Register styles if they are not part of the cache and attach them
         */
        this.registerStyles = (hashId, css) => {
            if (FASTStyle.supportsAdoptedStyleSheets) {
                if (!(hashId in FASTStyle.cache)) {
                    this.memoizeAdoptedStylesheetStyles(hashId, css);
                }
                this.attachAdoptedStylesheetStyles(this.parentNode, hashId);
            }
            else {
                if (!(hashId in FASTStyle.cache)) {
                    this.memoizeStyleElementStyles(hashId, css);
                }
                this.attachStyleElementStyles(this.parentNode, hashId);
            }
        };
    }
    /**
     * @internal
     */
    connectedCallback() {
        const hashId = this.getAttribute(FASTStyle.hashIdDataSetName);
        const css = this.getAttribute("css");
        this.registerStyles(hashId, css);
    }
    /**
     * Memoize CSSStyleSheets
     */
    memoizeAdoptedStylesheetStyles(hashId, css) {
        const sheet = new CSSStyleSheet();
        sheet.replaceSync(css);
        FASTStyle.cache[hashId] = sheet;
    }
    /**
     * Attach CSSStyleSheets
     */
    attachAdoptedStylesheetStyles(shadowRoot, hashId) {
        shadowRoot.adoptedStyleSheets = [
            ...shadowRoot.adoptedStyleSheets,
            FASTStyle.cache[hashId],
        ];
    }
    /**
     * Memoize css strings
     */
    memoizeStyleElementStyles(hashId, css) {
        FASTStyle.cache[hashId] = css;
    }
    /**
     * Attach style elements
     */
    attachStyleElementStyles(shadowRoot, hashId) {
        const element = document.createElement("style");
        element.innerHTML = FASTStyle.cache[hashId];
        shadowRoot.append(element);
    }
}
FASTStyle.cache = {};
FASTStyle.hashIdDataSetName = "style-id";
FASTStyle.supportsAdoptedStyleSheets = Array.isArray(document.adoptedStyleSheets) &&
    "replace" in CSSStyleSheet.prototype;
export const fastStyleTagName = "fast-style";
customElements.define(fastStyleTagName, FASTStyle);
