import { __decorate } from "tslib";
import "../install-dom-shim.js";
import { customElement, DOMAspect, FASTElement, html, ViewTemplate } from "@microsoft/fast-element";
import { expect, test } from "@playwright/test";
import { parseTemplateToOpCodes } from "./template-parser.js";
let HelloWorld = class HelloWorld extends FASTElement {
};
HelloWorld = __decorate([
    customElement("hello-world")
], HelloWorld);
function firstFactory(factories) {
    for (const key in factories) {
        return factories[key];
    }
    return null;
}
test.describe("parseTemplateToOpCodes", () => {
    test("should throw when invoked with a ViewTemplate with a HTMLTemplateElement template", () => {
        expect(() => {
            parseTemplateToOpCodes(new ViewTemplate(document.createElement("template"), {}));
        }).toThrow();
    });
    test("should not throw when invoked with a ViewTemplate with a string template", () => {
        expect(() => {
            parseTemplateToOpCodes(new ViewTemplate("", {}));
        }).not.toThrow();
    });
    test("should emit a single text op for a template with no bindings or directives", () => {
        expect(parseTemplateToOpCodes(html `<p>Hello world</p>`)).toEqual([{ type: 8 /* OpType.text */, value: "<p>Hello world</p>" }]);
    });
    test("should emit doctype, html, head, and body elements as part of text op", () => {
        expect(parseTemplateToOpCodes(html `<!DOCTYPE html><html><head></head><body></body></html>`)).toEqual([{ type: 8 /* OpType.text */, value: "<!DOCTYPE html><html><head></head><body></body></html>" }]);
    });
    test("should emit a viewBehaviorFactory op from a binding", () => {
        const input = html `${() => "hello world"}`;
        expect(parseTemplateToOpCodes(input)).toEqual([{ type: 5 /* OpType.viewBehaviorFactory */, factory: firstFactory(input.factories) }]);
    });
    test("should emit a directive op from a content binding", () => {
        const input = html `Hello ${() => "World"}.`;
        const codes = parseTemplateToOpCodes(input);
        const code = codes[1];
        expect(codes.length).toBe(3);
        expect(code.type).toBe(5 /* OpType.viewBehaviorFactory */);
    });
    test("should sandwich viewBehaviorFactory ops between text ops when binding native element content", () => {
        const input = html `<p>${() => "hello world"}</p>`;
        expect(parseTemplateToOpCodes(input)).toEqual([
            { type: 8 /* OpType.text */, value: "<p>" },
            { type: 5 /* OpType.viewBehaviorFactory */, factory: firstFactory(input.factories) },
            { type: 8 /* OpType.text */, value: "</p>" },
        ]);
    });
    test("should emit a custom element as text if it has not been defined", () => {
        const input = html `<undefined-element test-attribute="test" test-bool></undefined-element>`;
        expect(parseTemplateToOpCodes(input)).toEqual([{ type: 8 /* OpType.text */, value: "<undefined-element test-attribute=\"test\" test-bool></undefined-element>" }]);
    });
    test("should emit custom element open, close, attribute, and shadow ops for a defined custom element", () => {
        const input = html `<hello-world></hello-world>`;
        expect(parseTemplateToOpCodes(input)).toEqual([
            { type: 0 /* OpType.customElementOpen */, ctor: HelloWorld, tagName: "hello-world", staticAttributes: new Map() },
            { type: 8 /* OpType.text */, value: "<hello-world" },
            { type: 2 /* OpType.customElementAttributes */ },
            { type: 8 /* OpType.text */, value: ">" },
            { type: 3 /* OpType.customElementShadow */ },
            { type: 1 /* OpType.customElementClose */ },
            { type: 8 /* OpType.text */, value: "</hello-world>" }
        ]);
    });
    test("should emit static attributes of a custom element custom element open, close, attribute, and shadow ops for a defined custom element", () => {
        const input = html `<hello-world string-value="test" bool-value></hello-world>`;
        const code = parseTemplateToOpCodes(input).find((op) => op.type === 0 /* OpType.customElementOpen */);
        expect(code).not.toBeUndefined();
        expect(code === null || code === void 0 ? void 0 : code.staticAttributes.get("string-value")).toBe("test");
        expect(code === null || code === void 0 ? void 0 : code.staticAttributes.get("bool-value")).toBe("");
        expect(code === null || code === void 0 ? void 0 : code.staticAttributes.size).toBe(2);
    });
    test("should emit attributes binding ops for a native element with attribute bindings", () => {
        const input = html `<p string-value="${x => "value"}" ?bool-value="${x => false}" :property-value="${x => "value"}" @event="${x => { }}"></p>`;
        const codes = parseTemplateToOpCodes(input).filter(x => x.type === 4 /* OpType.attributeBinding */);
        expect(codes.length).toBe(4);
        expect(codes[0].aspect).toBe(DOMAspect.attribute);
        expect(codes[1].aspect).toBe(DOMAspect.booleanAttribute);
        expect(codes[2].aspect).toBe(DOMAspect.property);
        expect(codes[3].aspect).toBe(DOMAspect.event);
    });
    test("should emit attributes binding ops for a custom element with attribute bindings", () => {
        const input = html `<hello-world string-value="${x => "value"}" ?bool-value="${x => false}" :property-value="${x => "value"}" @event="${x => { }}"></hello-world>`;
        const codes = parseTemplateToOpCodes(input).filter(x => x.type === 4 /* OpType.attributeBinding */);
        expect(codes.length).toBe(4);
        expect(codes[0].aspect).toBe(DOMAspect.attribute);
        expect(codes[1].aspect).toBe(DOMAspect.booleanAttribute);
        expect(codes[2].aspect).toBe(DOMAspect.property);
        expect(codes[3].aspect).toBe(DOMAspect.event);
    });
    test("should emit names to attribute bindings ops that do not contain attribute type prefixes", () => {
        const input = html `<hello-world string-value="${x => "value"}" ?bool-value="${x => false}" :property-value="${x => "value"}" @event="${x => { }}"></hello-world>`;
        const codes = parseTemplateToOpCodes(input).filter(x => x.type === 4 /* OpType.attributeBinding */);
        expect(codes.length).toBe(4);
    });
    test("should emit template open and close ops for a template element", () => {
        const input = html `<template></template>`;
        const codes = parseTemplateToOpCodes(input);
        expect(codes.length).toBe(2);
        expect(codes[0].type).toBe(6 /* OpType.templateElementOpen */);
        expect(codes[1].type).toBe(7 /* OpType.templateElementClose */);
    });
    test("should emit template open ops with static attributes", () => {
        const input = html `<template id="foo" boolean></template>`;
        const open = parseTemplateToOpCodes(input)[0];
        expect(open.staticAttributes.get("id")).toBe("foo");
        expect(open.staticAttributes.get("boolean")).toBe("");
    });
    test("should emit template open ops with dynamic attributes", () => {
        const input = html `<template id=${x => "foo"} ?boolean=${x => true} @event=${x => undefined} :property=${x => "value"}></template>`;
        const open = parseTemplateToOpCodes(input)[0];
        const attrs = new Map(open.dynamicAttributes.map(x => {
            return [x.target, x];
        }));
        expect(attrs.has("id")).toBe(true);
        expect(attrs.get("id").aspect).toBe(DOMAspect.attribute);
        expect(attrs.has("boolean")).toBe(true);
        expect(attrs.get("boolean").aspect).toBe(DOMAspect.booleanAttribute);
        expect(attrs.has("event")).toBe(true);
        expect(attrs.get("event").aspect).toBe(DOMAspect.event);
        expect(attrs.has("property")).toBe(true);
        expect(attrs.get("property").aspect).toBe(DOMAspect.property);
    });
    test("should emit template open ops with static and dynamic attributes", () => {
        const input = html `<template id="foo" ?boolean=${x => true}></template>`;
        const open = parseTemplateToOpCodes(input)[0];
        expect(open.staticAttributes.size).toBe(1);
        expect(open.staticAttributes.get("id")).toBe("foo");
        expect(open.dynamicAttributes.length).toBe(1);
        expect(open.dynamicAttributes[0].target).toBe("boolean");
    });
    test("should emit template template ops between other ops when nested inside of another element", () => {
        const input = html `<div><template></template></div>`;
        const codes = parseTemplateToOpCodes(input);
        expect(codes[0].type).toBe(8 /* OpType.text */);
        expect(codes[0].value).toBe(`<div>`);
        expect(codes[1].type).toBe(6 /* OpType.templateElementOpen */);
        expect(codes[2].type).toBe(7 /* OpType.templateElementClose */);
        expect(codes[3].type).toBe(8 /* OpType.text */);
        expect(codes[3].value).toBe(`</div>`);
    });
});
